# This file is generated. Do not modify by hand.
from enum import Enum


class Errors(Enum):

    REQUEST_TIMEOUT = 0
    CONNECTION_CLOSED = 1
    INVALID_ARGUMENT = 2
    OUT_OF_REQUEST_IDS = 3
    INVALID_PACKET = 4
    CONNECTION_FAILED = 5
    UNKNOWN_REQUEST = 6
    INVALID_REQUEST_DATA = 7
    COMMAND_FAILED = 8
    DEVICE_DB_FAILED = 9
    INVALID_DATA = 10
    DEVICE_NOT_IDENTIFIED = 11
    CONVERSION_FAILED = 12
    DEVICE_ADDRESS_CONFLICT = 13
    NO_DEVICE_FOUND = 14
    MOVEMENT_INTERRUPTED = 15
    MOVEMENT_FAILED = 16
    IO_FAILED = 17
    INVALID_RESPONSE = 18
    NOT_SUPPORTED = 19
    DEVICE_FAILED = 20
    OS_FAILED = 21
    INTERNAL_ERROR = 22
    BINARY_COMMAND_FAILED = 23
    COMMAND_PREEMPTED = 24
    LOCKSTEP_NOT_ENABLED = 25
    LOCKSTEP_ENABLED = 26
    IO_CHANNEL_OUT_OF_RANGE = 27
    SETTING_NOT_FOUND = 28
    STREAM_MODE = 29
    STREAM_EXECUTION = 30
    STREAM_MOVEMENT_FAILED = 31
    STREAM_SETUP_FAILED = 32
    DEVICE_BUSY = 33
    STREAM_MOVEMENT_INTERRUPTED = 34
    INVALID_PARK_STATE = 35
    SERIAL_PORT_BUSY = 36
    TRANSPORT_ALREADY_USED = 37
    SET_PERIPHERAL_STATE_FAILED = 38
    SET_DEVICE_STATE_FAILED = 39
    G_CODE_SYNTAX = 40
    G_CODE_EXECUTION = 41
    INVALID_OPERATION = 42
    COMMAND_TOO_LONG = 43
    NO_VALUE_FOR_KEY = 44
    DEVICE_DETECTION_FAILED = 45
    TIMEOUT = 46
    PVT_MODE = 47
    PVT_EXECUTION = 48
    PVT_SETUP_FAILED = 49
    PVT_MOVEMENT_FAILED = 50
    PVT_MOVEMENT_INTERRUPTED = 51
    PVT_DISCONTINUITY = 52
    STREAM_DISCONTINUITY = 53
    OPERATION_FAILED = 54
    INCOMPATIBLE_SHARED_LIBRARY = 55
