# This file is generated. Do not modify by hand.
from enum import Enum


class TriggerCondition(Enum):
    """
    Comparison operator for trigger condition.
    """

    EQ = 0
    NE = 1
    GT = 2
    GE = 3
    LT = 4
    LE = 5
